/**
 * @file crfmodel_graphcut.h
 * @author  Iulian Vlad Serbane <julianserban@gmail.com>
 * @version 1.0
 * @section LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details at
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef CRFMODEL_GRAPHCUT_H
#define CRFMODEL_GRAPHCUT_H

#include <crfmodel.h>

#include <map>
#include <random>

#include <QDebug>

#include <math.h>
#include "include/graph.h"

/*#include <boost/config.hpp>
#include <iostream>
#include <string>
#include <boost/graph/push_relabel_max_flow.hpp>
#include <boost/graph/edmonds_karp_max_flow.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/connected_components.hpp>
*/

namespace CRF
{
    class CRFModel_GraphCut : public CRFModel
    {
    public:
        /**
         * Constructor that initializes the configuration of the model. Width and Height are set to 1 pixel.
         */
        CRFModel_GraphCut();

        /**
         * Constructor that initializes the configuration of the model.
         * @param imgWidth Image width in pixels.
         * @param imgHeight Image height in pixels.
         * @param MAPSTEPS Maximum number of steps to perform for MAP estimation. This is irrelevant for Graph Cut methods.
         */
        CRFModel_GraphCut(int imgWidth, int imgHeight, int MAPSTEPS);

        /**
         * Finds the MAP (Maximum Aposterori Estimation) solution using Graph Cut methods.
         * @param Input Input sample.
         * @return MAP solution.
         */
        int_int_vector EstimateMAP(const double_double_vector Input);


    private:

    };
}
#endif // CRFMODEL_GRAPHCUT_H
