#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QDirIterator>
#include <QFile>
#include <QDebug>
#include <QPixmap>

using namespace std;
using namespace CRF;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    ui->EstimateParametersButton->setEnabled(false);
    ui->DenoiseImageButton->setEnabled(false);
    ui->ZoomFactorLineEdit->setEnabled(false);

    ui->IterationStepsLineEdit->setEnabled(false);
    ui->StepFactorLineEdit->setEnabled(false);

    ui->CRFModelComboBox->addItem("Exact Likelihood");
    ui->CRFModelComboBox->addItem("Constrastive Divergence");
    ui->CRFModelComboBox->setCurrentIndex(0);

    ui->CRFModelComboBox->setEditable(true);
    ui->CRFModelComboBox->lineEdit()->setAlignment(Qt::AlignCenter);
}

MainWindow::~MainWindow()
{
    delete ui;
}

// mainwindow.cpp
void MainWindow::LoadDataButtonClicked()
{
    int SelIndex = ui->CRFModelComboBox->currentIndex();
    if (SelIndex == 1)
    {
        UseCDModel = true;
    }
    else
    {
        UseCDModel = false;
    }


    int PixelsWidth = ui->ImagePixelsWidthLineEdit->text().toInt();
    int PixelsHeight = ui->ImagePixelsHeightLineEdit->text().toInt();

    //Load the data
    QString_vector Fls = LoadData(ui->InputSamplesFolderLineEdit->text(), ui->OutputSamplesFolderLineEdit->text(), PixelsWidth, PixelsHeight);

    //Load data into interface
    for (size_t i = 0; i < Fls.size(); i++)
    {
        ui->InputFilesListWidget->addItem(Fls[i]);
    }

    //Change interface...
    ui->EstimateParametersButton->setEnabled(true);

    ui->ImagePixelsWidthLineEdit->setEnabled(false);
    ui->ImagePixelsHeightLineEdit->setEnabled(false);

    ui->InputSamplesFolderLineEdit->setEnabled(false);
    ui->OutputSamplesFolderLineEdit->setEnabled(false);

    ui->GibbsChainsLineEdit->setEnabled(false);
    ui->GibbsChainStepsLineEdit->setEnabled(false);
    ui->MAPIterationsLineEdit->setEnabled(false);

    ui->LoadDataButton->setEnabled(false);

    ui->IterationStepsLineEdit->setEnabled(true);
    ui->StepFactorLineEdit->setEnabled(true);
}

//Loads the files and returns their names (only the names, not the full path though)
QString_vector MainWindow::LoadData(QString inputSamplesDir, QString outputSamplesDir, int pixelsWidth, int pixelsHeight)
{
    QString_vector Ret;

    QDirIterator directory_walker(inputSamplesDir, QDir::Files | QDir::NoSymLinks);
    QFile file_checker;

    std::vector<QString> InputFilenameArray;
    std::vector<QString> OutputFilenameArray;
    InputFilenameArray.reserve(128);
    OutputFilenameArray.reserve(128);

   //Loop through the directory and list all image files
   while(directory_walker.hasNext())
   {
         directory_walker.next();

        //We are only working with png files
        if(directory_walker.fileInfo().completeSuffix() == "png")
        {
            QString CorrespondingOutputFile;
            CorrespondingOutputFile = outputSamplesDir + "/" + directory_walker.fileName();

            if (file_checker.exists(CorrespondingOutputFile) == true)
            {
                QString Currfil = directory_walker.fileName();
                InputFilenameArray.push_back(inputSamplesDir + "/" + Currfil);
                OutputFilenameArray.push_back(CorrespondingOutputFile);
                Ret.push_back(Currfil);
            }
            else
            {
                qDebug() << "No corresponding output sample for input sample " << directory_walker.fileName();
            }
        }
   }

   int imageCount = InputFilenameArray.size();
   OutputSamplesArray = std::vector<int_int_vector>(imageCount);
   InputSamplesArray = std::vector<double_double_vector>(imageCount);

   for (size_t i = 0; i < InputFilenameArray.size(); i++)
   {
       InputSamplesArray[i] = CRF_GetInputSampleFromImage(InputFilenameArray[i],pixelsWidth,pixelsHeight);
       OutputSamplesArray[i] = CRF_GetOutputSampleFromImage(OutputFilenameArray[i],pixelsWidth,pixelsHeight);
   }

   return Ret;
}

void MainWindow::EstimateParametersButtonClicked()
{
    int PixelsWidth = ui->ImagePixelsWidthLineEdit->text().toInt();
    int PixelsHeight = ui->ImagePixelsWidthLineEdit->text().toInt();
    size_t IterationSteps = ui->IterationStepsLineEdit->text().toInt();
    double StepFactor = ui->StepFactorLineEdit->text().toDouble();

    int MapIt =  ui->MAPIterationsLineEdit->text().toInt();

    qDebug() << "Initializing CRFModel...";
    CRFMod = CRFModel_GraphCut(PixelsWidth,PixelsHeight,MapIt);
    CRFMod_CD = CRFModel_CD(PixelsWidth,PixelsHeight, MapIt);

    int GibbsChains =  ui->GibbsChainsLineEdit->text().toInt();
    int GibbsIt =  ui->GibbsChainStepsLineEdit->text().toInt();

    CRFMod_CD.SetCDParameters(GibbsIt, GibbsChains);

    //Used for experiments:
    /*qDebug() << "Estimating Model Parameters... CD-1";
    for (int i = 0; i < 100; i++)
    {
        qDebug() << "Iteration: " << i + 1 << " out of 100, \eta = 25";
        CRFMod_CD.chainSteps = 1;
        CRFMod_CD.testNumber = 0;
        CRFMod_CD.EstimateModelParameters(InputSamplesArray, OutputSamplesArray, IterationSteps, 10);
    }

    qDebug() << "Estimating Model Parameters... CD-1, \eta=10";
    for (int i = 0; i < 100; i++)
    {
        qDebug() << "Iteration: " << i + 1 << " out of 100";
        CRFMod_CD.chainSteps = 1;
        CRFMod_CD.testNumber = 0;
        CRFMod_CD.EstimateModelParameters(InputSamplesArray, OutputSamplesArray, IterationSteps, 10);
    }

    qDebug() << "Estimating Model Parameters... CD-20 \eta=10";
    for (int i = 0; i < 100; i++)
    {
        qDebug() << "Iteration: " << i + 1 << " out of 100";

        CRFMod_CD.chainSteps = 20;
        CRFMod_CD.testNumber = 0;
        CRFMod_CD.EstimateModelParameters(InputSamplesArray, OutputSamplesArray, IterationSteps, 10);
    }*/

    qDebug() << "Training CRFModel...";
    if (UseCDModel == true)
    {
        CRFMod_CD.chainSteps = 1;
        CRFMod_CD.EstimateModelParameters(InputSamplesArray, OutputSamplesArray, IterationSteps, StepFactor);
    }
    else
    {
        CRFMod.EstimateModelParameters(InputSamplesArray, OutputSamplesArray, IterationSteps, StepFactor);
    }

    qDebug() << "Finished!";

    ui->EstimateParametersButton->setEnabled(false);
    ui->DenoiseImageButton->setEnabled(true);
    ui->ZoomFactorLineEdit->setEnabled(true);

    ui->IterationStepsLineEdit->setEnabled(false);
    ui->StepFactorLineEdit->setEnabled(false);
}

void MainWindow::DenoiseImageButtonClicked()
{
    int ZoomFactor = ui->ZoomFactorLineEdit->text().toInt();

    int PixelsWidth = ui->ImagePixelsWidthLineEdit->text().toInt();
    int PixelsHeight = ui->ImagePixelsWidthLineEdit->text().toInt();

    QList<QListWidgetItem*> ItemsList = ui->InputFilesListWidget->selectedItems();
    if (ItemsList.size() < 1)
    {
        return ; //No items selected
    }

    //Get input / correct output data
    QString InputFile = ui->InputSamplesFolderLineEdit->text() + "/" + ItemsList[0]->text();
    QString OutputFile =ui->OutputSamplesFolderLineEdit->text() + "/" + ItemsList[0]->text();

    double_double_vector CRF_Model_Input = CRF_GetInputSampleFromImage(InputFile, PixelsWidth, PixelsHeight);
    int_int_vector CRF_Model_Correct_Output = CRF_GetOutputSampleFromImage(OutputFile, PixelsWidth, PixelsHeight);

    //Draw the input image and the correct output image...
    QImage InputImg(PixelsWidth, PixelsHeight, QImage::Format_RGB32);

    for (int a = 0; a < PixelsWidth; a++)
    {
        for (int b = 0; b < PixelsHeight; b++)
         {
             double ScaledInp = ((1 + CRF_Model_Input[a][b]) / 2) * 255;
             int ScaledInd = 255 - ScaledInp;
             QColor newcol(ScaledInd, ScaledInd, ScaledInd, 255);

             InputImg.setPixel(a, b, newcol.rgb());
         }
    }

    QRgb blackpx = qRgb(0,0,0);
    QRgb whitepx = qRgb(255,255,255);

    QPixmap FinalInputImg(QPixmap::fromImage(InputImg));
    QPixmap ScaledInputImg = FinalInputImg.scaled(PixelsWidth*ZoomFactor, PixelsHeight*ZoomFactor);
    ui->NoisyImageImgLabel->setPixmap(ScaledInputImg);

    QImage OutputImg(PixelsWidth, PixelsHeight, QImage::Format_RGB32);
    for (int a = 0; a < PixelsWidth; a++)
    {
        for (int b = 0; b < PixelsHeight; b++)
         {
             OutputImg.setPixel(a, b, whitepx);

             if (CRF_Model_Correct_Output[a][b] == 1)
              {
                  OutputImg.setPixel(a, b, blackpx);
              }

         }
    }

    QPixmap FinalOutputImg(QPixmap::fromImage(OutputImg));
    QPixmap ScaledOutputImg = FinalOutputImg.scaled(PixelsWidth*ZoomFactor, PixelsHeight*ZoomFactor);
    ui->CorrectlyDenoisedImageImgLabel->setPixmap(ScaledOutputImg);

    //Get the CRF denoised image
    int_int_vector CRF_Model_Output;
    if (UseCDModel == true)
    {
        CRF_Model_Output = CRFMod_CD.EstimateMAP(CRF_Model_Input);
    }
    else
    {
        CRF_Model_Output = CRFMod.EstimateMAP(CRF_Model_Input);
    }

    //Draw the CRF denoised image
    QImage CRFImg(PixelsWidth, PixelsHeight, QImage::Format_RGB32);

    for (int a = 0; a < PixelsWidth; a++)
    {
        for (int b = 0; b < PixelsHeight; b++)
         {
             CRFImg.setPixel(a, b, whitepx);

             if (CRF_Model_Output[a][b] == 1)
             {
                 CRFImg.setPixel(a, b, blackpx);
             }
         }
    }

    QPixmap FinalCRFImg(QPixmap::fromImage(CRFImg));
    QPixmap ScaledCRFImg = FinalCRFImg.scaled(PixelsWidth*ZoomFactor, PixelsHeight*ZoomFactor);
    ui->CRFDenoisedImageImgLabel->setPixmap(ScaledCRFImg);
}

void MainWindow::ResetButtonClicked()
{
    ui->EstimateParametersButton->setEnabled(false);
    ui->DenoiseImageButton->setEnabled(false);
    ui->ZoomFactorLineEdit->setEnabled(false);

    ui->ImagePixelsWidthLineEdit->setEnabled(true);
    ui->ImagePixelsHeightLineEdit->setEnabled(true);

    ui->InputSamplesFolderLineEdit->setEnabled(true);
    ui->OutputSamplesFolderLineEdit->setEnabled(true);

    ui->LoadDataButton->setEnabled(true);

    ui->IterationStepsLineEdit->setEnabled(false);
    ui->StepFactorLineEdit->setEnabled(false);

    ui->GibbsChainsLineEdit->setEnabled(true);
    ui->GibbsChainStepsLineEdit->setEnabled(true);
    ui->MAPIterationsLineEdit->setEnabled(true);

    while(ui->InputFilesListWidget->count() > 0)
    {
        ui->InputFilesListWidget->takeItem(0);
    }
}

int_int_vector MainWindow:: CRF_GetOutputSampleFromImage(QString ImageFile, int pixelsWidth, int pixelsHeight)
{
    QImage myImage;
    myImage.load(ImageFile);

    if (myImage.width() < pixelsWidth || myImage.height() < pixelsHeight)
    {
        QString T("Image size is incorrect for image " + ImageFile + ".");
        throw T;
    }

    int_int_vector Ret(pixelsWidth);
    for (int a = 0; a < pixelsWidth; a++)
    {
       Ret[a] = int_vector(pixelsHeight);
       for (int b = 0; b < pixelsHeight; b++)
       {
           int luminance = qGray(myImage.pixel(a,b));

           if (luminance > 100)
           {
               Ret[a][b] = 0;
           }
           else
           {
                Ret[a][b] = 1;
           }
       }
    }

    return Ret;
}

double_double_vector MainWindow:: CRF_GetInputSampleFromImage(QString ImageFile, int pixelsWidth, int pixelsHeight)
{
    QImage myImage;
    myImage.load(ImageFile);

    if (myImage.width() < pixelsWidth || myImage.height() < pixelsHeight)
    {
        QString T("Image size is incorrect for image " + ImageFile + ".");
        throw T;
    }

    double_double_vector Ret(pixelsWidth);
    for (int a = 0; a < pixelsWidth; a++)
    {
       Ret[a] = double_vector(pixelsHeight);
       for (int b = 0; b < pixelsHeight; b++)
       {
           //This is the opposite of the description in the thesis, but will give the exact same result for the model.
           double grayness = 255 - qGray(myImage.pixel(a,b));

           double grayness_level = grayness / 255;
           Ret[a][b] = grayness_level*2 - 1;
           //qDebug() << "InputImage[" << a << "][" << b << "]: " << Ret[a][b];
       }
    }

    return Ret;
}
