/**
 * @file crfmodel_cd.h
 * @author  Iulian Vlad Serbane <julianserban@gmail.com>
 * @version 1.0
 * @section LICENSE
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details at
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>

#include <crfmodel.h>
#include <crfmodel_cd.h>
#include <crfmodel_graphcut.h>

using namespace CRF;

typedef std::vector<QString> QString_vector;

typedef std::vector<double> double_vector;
typedef std::vector< std::vector<double> > double_double_vector;

typedef std::vector<size_t> int_vector;
typedef std::vector< std::vector<size_t> > int_int_vector;

namespace Ui {
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
public slots:
    void LoadDataButtonClicked();
    void EstimateParametersButtonClicked();
    void DenoiseImageButtonClicked();
    void ResetButtonClicked();
private:
    bool UseCDModel;
    CRFModel_GraphCut CRFMod;
    CRFModel_CD CRFMod_CD;

    std::vector<int_int_vector> OutputSamplesArray;
    std::vector<double_double_vector> InputSamplesArray;

    Ui::MainWindow *ui;

    QString_vector LoadData(QString inputSamplesDir, QString outputSamplesDir, int pixelsWidth, int pixelsHeight);
    int_int_vector CRF_GetOutputSampleFromImage(QString ImageFile, int pixelsWidth, int pixelsHeight);
    double_double_vector CRF_GetInputSampleFromImage(QString ImageFile, int pixelsWidth, int pixelsHeight);
};


#endif // MAINWINDOW_H
